#!/usr/bin/env python3
###############################################################################
# Orkid Build System
# Copyright 2010-2018, Michael T. Mayers
# email: michael@tweakoz.com
# The Orkid Build System is published under the GPL 2.0 license
# see http://www.gnu.org/licenses/gpl-2.0.html
###############################################################################

import argparse, sys, os, json
from ork import env, path

parser = argparse.ArgumentParser(description='ork.build environment launcher')
parser.add_argument("-w", '--working-dir', metavar="workdir", help='launch from pre-existing folder' )
parser.add_argument("-p", '--project-dir', metavar="prjdir", help='override project directory' )
parser.add_argument("-c", '--command', metavar="command", help="execute in environ")
parser.add_argument("-x", '--subspace', metavar="subspace", default ="host", help='subspace to launch' )
parser.add_argument("-o", '--output', metavar="output", help='output json file' )

args = vars(parser.parse_args())

# get env vars from os.environ

PATH = os.environ["PATH"]
ORKID_WORKSPACE_DIR = os.environ["ORKID_WORKSPACE_DIR"]
OBT_STAGE = os.environ["OBT_STAGE"]
OBT_STAGE = os.environ["OBT_STAGE"]
OBT_ROOT = os.environ["OBT_ROOT"]
OBT_SUBSPACE = os.environ["OBT_SUBSPACE"]
LD_LIBRARY_PATH = os.environ["LD_LIBRARY_PATH"]

# create vs code launch json data structure for debug, including env vars

launch_json = {
    "version": "0.2.0",
    "configurations": [
        {
            "name": "ork.build",
            "type": "cppdbg",
            "request": "launch",
            "program": "${workspaceFolder}/ork.build/bin/obt",
            "args": [
                "--launch",
                "${workspaceFolder}",
                "--subspace",
                "host",
                "--command",
                "bash"
            ],
            "stopAtEntry": False,
            "cwd": "${workspaceFolder}",
            "environment": []

        }
    ]
}

# add env vars to launch json

for item in [
    "PATH",
    "ORKID_WORKSPACE_DIR",
    "OBT_STAGE",
    "OBT_STAGE",
    "OBT_ROOT",
    "OBT_SUBSPACE",
    "LD_LIBRARY_PATH"
]:
    launch_json["configurations"][0]["environment"].append({
        "name": item,
        "value": os.environ[item]
    })  

# write launch json to file

if args["output"]!=None:
    with open(args["output"], 'w') as outfile:
        json.dump(launch_json, outfile, indent=4)




###############################################################################
# Orkid Build System
# Copyright 2010-2020, Michael T. Mayers
# email: michael@tweakoz.com
# The Orkid Build System is published under the GPL 2.0 license
# see http://www.gnu.org/licenses/gpl-2.0.html
###############################################################################

from obt import dep, path, host

###############################################################################
# libtorch is quarantined for now..
###############################################################################

class libtorch(dep.StdProvider):
  name = "libtorch"
  def __init__(self):
    super().__init__(libtorch.name)
    self.declareDep("cmake")
    self.declareDep("zmq")
    self.declareDep("ffmpeg")
    PYTHON = dep.instance("python")

    self._builder = self.createBuilder(dep.CMakeBuilder,install_prefix=self.prefix)
    self._builder.setCmVar("PYTHON_EXECUTABLE",PYTHON.executable)
    self._builder.setCmVar("USE_NCCL","OFF") # NVIDIA Collective Communication Library (causes errors, atm....)
    self._builder.setCmVar("USE_CUDA","OFF") # (causes errors, atm....)
    self._builder.setCmVar("USE_ZMQ","ON") #
    self._builder.setCmVar("USE_FFMPEG","ON") #

    if host.IsOsx:
      self._builder.setCmVar("USE_METAL", "ON")
      self._builder.setCmVar("CMAKE_CXX_FLAGS", "-Wno-unknown-warning-option")
    else:
      self._builder.setCmVar("BUILD_CAFFE2","ON") #

  ########################################################################
  @property
  def _fetcher(self):
    return dep.GithubFetcher(name=libtorch.name,
                             repospec="pytorch/pytorch",
                             revision="v1.12.0",
                             recursive=True)
  ########################################################################
  @property
  def prefix(self):
    return path.quarantine()/"libtorch"
  ########################################################################
  def areRequiredSourceFilesPresent(self):
    return (self.source_root/"CMakeLists.txt").exists()
  def areRequiredBinaryFilesPresent(self):
    return (self.prefix/"lib"/"liblibtorchx.so").exists()




"""

Caffe2 is a deep learning framework.that provides an easy and straightforward way for you to experiment
 with deep learning and leverage community contributions of new models and algorithms. You can bring your 
 creations to scale using the power of GPUs in the cloud or to the masses on mobile 
 with Caffe2’s cross-platform libraries.

--   Build type            : Release
--   Compile definitions   : ONNX_ML=1;ONNXIFI_ENABLE_EXT=1;ONNX_NAMESPACE=onnx_torch;HAVE_MMAP=1;_FILE_OFFSET_BITS=64;HAVE_SHM_OPEN=1;HAVE_SHM_UNLINK=1;HAVE_MALLOC_USABLE_SIZE=1;USE_EXTERNAL_MZCRC;MINIZ_DISABLE_ZIP_READER_CRC32_CHECKS
--   CMAKE_PREFIX_PATH     : /usr
--   USE_GOLD_LINKER       : OFF
-- 
--   TORCH_VERSION         : 1.12.0
--   CAFFE2_VERSION        : 1.12.0
--   BUILD_CAFFE2          : OFF
--   BUILD_CAFFE2_OPS      : OFF
--   BUILD_CAFFE2_MOBILE   : OFF
--   BUILD_STATIC_RUNTIME_BENCHMARK: OFF
--   BUILD_TENSOREXPR_BENCHMARK: OFF
--   BUILD_NVFUSER_BENCHMARK: OFF
--   BUILD_BINARY          : OFF
--   BUILD_CUSTOM_PROTOBUF : ON
--     Link local protobuf : ON
--   BUILD_DOCS            : OFF
--   BUILD_PYTHON          : ON
--     Python version      : 3.9.13
--     Python executable   : <staging>/pyvenv/bin/python3
--     Pythonlibs version  : 3.9.13
--     Python library      : <staging>/python-3.9.13/lib/python3.9
--     Python includes     : <staging>/python-3.9.13/include/python3.9
--     Python site-packages: lib/python3.9/site-packages
--   BUILD_SHARED_LIBS     : ON
--   CAFFE2_USE_MSVC_STATIC_RUNTIME     : OFF
--   BUILD_TEST            : OFF
--   BUILD_JNI             : OFF
--   BUILD_MOBILE_AUTOGRAD : OFF
--   BUILD_LITE_INTERPRETER: OFF
--   INTERN_BUILD_MOBILE   : 
--   USE_BLAS              : 1
--     BLAS                : open
--     BLAS_HAS_SBGEMM     : 
--   USE_LAPACK            : 1
--     LAPACK              : open
--   USE_ASAN              : OFF
--   USE_CPP_CODE_COVERAGE : OFF
--   USE_CUDA              : ON
--     Split CUDA          : OFF
--     CUDA static link    : OFF
--     USE_CUDNN           : OFF
--     USE_EXPERIMENTAL_CUDNN_V8_API: ON
--     CUDA version        : 11.5
--     CUDA root directory : /usr
--     CUDA library        : /usr/lib/x86_64-linux-gnu/libcuda.so
--     cudart library      : /usr/lib/x86_64-linux-gnu/libcudart.so
--     cublas library      : /usr/lib/x86_64-linux-gnu/libcublas.so
--     cufft library       : /usr/lib/x86_64-linux-gnu/libcufft.so
--     curand library      : /usr/lib/x86_64-linux-gnu/libcurand.so
--     nvrtc               : /usr/lib/x86_64-linux-gnu/libnvrtc.so
--     CUDA include path   : /usr/include
--     NVCC executable     : /usr/bin/nvcc
--     CUDA compiler       : /usr/bin/nvcc

--     CUDA flags          :  -Xfatbin -compress-all -DONNX_NAMESPACE=onnx_torch -gencode arch=compute_86,code=sm_86 -Xcudafe --diag_suppress=cc_clobber_ignored,--diag_suppress=integer_sign_change,--diag_suppress=useless_using_declaration,--diag_suppress=set_but_not_used,--diag_suppress=field_without_dll_interface,--diag_suppress=base_class_has_different_dll_interface,--diag_suppress=dll_interface_conflict_none_assumed,--diag_suppress=dll_interface_conflict_dllexport_assumed,--diag_suppress=implicit_return_from_non_void_function,--diag_suppress=unsigned_compare_with_zero,--diag_suppress=declared_but_not_referenced,--diag_suppress=bad_friend_decl --expt-relaxed-constexpr --expt-extended-lambda  -Wno-deprecated-gpu-targets --expt-extended-lambda -DCUB_WRAPPED_NAMESPACE=at_cuda_detail -DCUDA_HAS_FP16=1 -D__CUDA_NO_HALF_OPERATORS__ -D__CUDA_NO_HALF_CONVERSIONS__ -D__CUDA_NO_HALF2_OPERATORS__ -D__CUDA_NO_BFLOAT16_CONVERSIONS__
--     CUDA host compiler  : 
--     CUDA --device-c     : OFF
--     USE_TENSORRT        : OFF
--   USE_ROCM              : OFF
--   USE_EIGEN_FOR_BLAS    : ON
--   USE_FBGEMM            : ON
--     USE_FAKELOWP          : OFF
--   USE_KINETO            : ON
--   USE_FFMPEG            : OFF
--   USE_GFLAGS            : OFF
--   USE_GLOG              : OFF
--   USE_LEVELDB           : OFF
--   USE_LITE_PROTO        : OFF
--   USE_LMDB              : OFF
--   USE_METAL             : OFF
--   USE_PYTORCH_METAL     : OFF
--   USE_PYTORCH_METAL_EXPORT     : OFF
--   USE_MPS               : OFF
--   USE_FFTW              : OFF
--   USE_MKL               : OFF
--   USE_MKLDNN            : ON
--   USE_NCCL              : ON
--     USE_SYSTEM_NCCL     : OFF
--     USE_NCCL_WITH_UCC   : OFF
--   USE_NNPACK            : ON
--   USE_NUMPY             : ON
--   USE_OBSERVERS         : ON
--   USE_OPENCL            : OFF
--   USE_OPENCV            : OFF
--   USE_OPENMP            : ON
--   USE_TBB               : OFF
--   USE_VULKAN            : OFF
--   USE_PROF              : OFF
--   USE_QNNPACK           : ON
--   USE_PYTORCH_QNNPACK   : ON
--   USE_XNNPACK           : ON
--   USE_REDIS             : OFF
--   USE_ROCKSDB           : OFF
--   USE_ZMQ               : OFF
--   USE_DISTRIBUTED       : ON
--     USE_MPI               : ON
--     USE_GLOO              : ON
--     USE_GLOO_WITH_OPENSSL : OFF
--     USE_TENSORPIPE        : ON
--   USE_DEPLOY           : OFF
--   Public Dependencies  : caffe2::Threads
--   Private Dependencies : pthreadpool;cpuinfo;qnnpack;pytorch_qnnpack;nnpack;XNNPACK;fbgemm;/usr/lib/x86_64-linux-gnu/libnuma.so;fp16;/usr/lib/x86_64-linux-gnu/libmpi_cxx.so;/usr/lib/x86_64-linux-gnu/libmpi.so;tensorpipe;gloo;foxi_loader;rt;fmt::fmt-header-only;kineto;gcc_s;gcc;dl
--   USE_COREML_DELEGATE     : OFF
--   BUILD_LAZY_TS_BACKEND   : ON

https://github.com/pytorch/pytorch/pull/16242
https://pytorch.org/tutorials/advanced/super_resolution_with_caffe2.html
https://pytorch.org/tutorials/advanced/cpp_export.html

"""



import obt.deco
import obt.path
import obt.template
from obt.command import Command
import fileinput

deco = obt.deco.Deco()

class patcher:

  def __init__(self,provider,repl_dict=None):
    name = provider._name
    self._ori = obt.path.patches(provider)/name/"ori"
    self._chg = obt.path.patches(provider)/name/"chg"
    self._repl_dict = repl_dict

  def patch(self,dest_dir,file):
    src  = self._chg/file
    dest = dest_dir/file
    print("Patching <%s -> %s>" % (deco.bright(src), deco.yellow(dest)))
    Command(["cp","-f",src,dest]).exec()
    if self._repl_dict:
      obt.template.template_file(dest,self._repl_dict)

  def patch_list(self,list_of_items):
    for i in list_of_items:
      print(i)
      self.patch(i[0],i[1])


def patch_with_dict(filename,item_dict):
  for k in item_dict.keys():
    v = item_dict[k]
    with fileinput.FileInput(str(filename), inplace=True, backup='.bak') as file:
      for line in file:
        print(line.replace(k, v), end='')


def patch_with_diffstr(file_path, diff_str):
    import os
    import subprocess
    import tempfile
    """
    Applies a unified diff (diff_str) to the specified file_path using the 'patch' command.

    :param diff_str: A string containing the unified diff.
    :param file_path: The path to the file that the diff should be applied to.
    :raises subprocess.CalledProcessError: If 'patch' exits with a non-zero status.
    """
    # Create a temporary file to store the diff
    with tempfile.NamedTemporaryFile(mode='w', delete=False) as temp_patch:
        temp_patch.write(diff_str)
        temp_patch.flush()
        temp_patch_path = str(temp_patch.name)

    try:
        # Call the system 'patch' command
        subprocess.check_call(["patch", str(file_path), temp_patch_path])
    finally:
        # Clean up the temporary file
        if os.path.exists(temp_patch_path):
            os.remove(temp_patch_path)



"""
C++ Entity Display Formatter
Provides clean formatting for C++ entities in search results
"""

import json
from typing import Dict, List, Optional, Set, Tuple, Any
from pathlib import Path

class TaggedString:
    """A string with associated formatting tags - pure data structure"""
    def __init__(self):
        self.parts = []  # List of (text, tag) tuples
    
    def add(self, text: str, tag: str = None):
        """Add text with optional tag"""
        self.parts.append((text, tag))
    
    def plain_text(self) -> str:
        """Get plain text without formatting"""
        return ''.join(text for text, tag in self.parts)

class CppFormatter:
    """Formats C++ entities for readable display"""
    
    def __init__(self, deco=None):
        self.typedef_cache = {}  # Cache for typedef/alias resolution
        self.namespace_context = ""  # Current namespace context
        self.deco = deco  # Decorator for colors (optional)
        
    def format_function_signature(self, entity) -> TaggedString:
        """
        Format a function signature for display
        
        Args:
            entity: Function entity with return_type and parameters
        
        Returns:
            TaggedString with formatted signature
        """
        result = TaggedString()
        
        # Get basic function name
        func_name = entity.name
        
        # Get return type
        return_type = getattr(entity, 'return_type', '')
        
        # Add virtual/static prefix if present
        if hasattr(entity, 'is_virtual') and entity.is_virtual:
            result.add("virtual ", None)
        elif hasattr(entity, 'is_static') and entity.is_static:
            result.add("static ", None)
            
        # Clean up and add return type
        if return_type:
            return_type = self._simplify_type(return_type, entity.namespace)
            result.add(return_type, 'return_type')
            result.add(" ", None)
            
        # Add function name
        result.add(func_name, 'function_name')
        
        # Parse and add parameters
        result.add("(", None)
        if hasattr(entity, 'parameters') and entity.parameters:
            try:
                if isinstance(entity.parameters, str):
                    params = json.loads(entity.parameters)
                else:
                    params = entity.parameters
                    
                # Calculate indent for wrapped params (align with first param)
                # This is the length of everything before the opening paren + 1
                indent_len = len(result.plain_text())
                indent = " " * indent_len
                
                # Format each parameter - first on same line, rest on new lines
                for i, param in enumerate(params):
                    if i > 0:
                        # New line with proper indent for subsequent params
                        result.add("\n" + indent, None)
                    self._add_parameter_to_result(result, param, entity.namespace)
                    # Add comma after each param except the last
                    if i < len(params) - 1:
                        result.add(",", None)
                    
            except (json.JSONDecodeError, TypeError):
                # Fallback to raw string if not valid JSON
                result.add(entity.parameters if entity.parameters else "", None)
        result.add(")", None)
        
        # Add const qualifier if present
        if hasattr(entity, 'is_const') and entity.is_const:
            result.add(" const", None)
            
        return result
    
    def _add_parameter_to_result(self, result: TaggedString, param: Dict, func_namespace: str):
        """Add a single parameter to the TaggedString result"""
        param_type = param.get('type', '')
        param_name = param.get('name', '')
        param_default = param.get('default', '')
        
        # Simplify the type
        param_type = self._simplify_type(param_type, func_namespace)
        
        # Add parameter type
        result.add(param_type, 'param_type')
        
        # Add parameter name if present
        if param_name:
            result.add(" ", None)
            result.add(param_name, 'param_name')
            
        # Add default value if present
        if param_default:
            result.add(" = ", None)
            result.add(param_default, None)
    
    def _simplify_type(self, type_str: str, current_namespace: str) -> str:
        """
        Simplify a type name based on context
        
        - Remove redundant namespace prefixes
        - Use typedef/alias names when available
        - Clean up template syntax
        """
        if not type_str:
            return type_str
            
        # First, check typedef cache
        if type_str in self.typedef_cache:
            type_str = self.typedef_cache[type_str]
            
        # Handle namespace-relative names
        if current_namespace:
            # Remove current namespace prefix if present
            ns_prefix = current_namespace + "::"
            if type_str.startswith(ns_prefix):
                type_str = type_str[len(ns_prefix):]
                
            # Also handle nested namespaces
            # e.g., if we're in ork::lev2 and type is ork::lev2::gfx::Buffer
            # we can simplify to gfx::Buffer
            parts = current_namespace.split("::")
            for i in range(len(parts), 0, -1):
                prefix = "::".join(parts[:i]) + "::"
                if type_str.startswith(prefix):
                    type_str = type_str[len(prefix):]
                    break
        
        # Clean up common patterns
        type_str = self._apply_common_simplifications(type_str)
        
        return type_str
    
    def _apply_common_simplifications(self, type_str: str) -> str:
        """Apply common type simplifications"""
        
        # Common typedef replacements (can be extended based on project conventions)
        common_typedefs = {
            # Orkid common typedefs
            "std::shared_ptr<Context>": "context_ptr_t",
            "std::shared_ptr<const Context>": "const_context_ptr_t",
            "std::shared_ptr<Buffer>": "buffer_ptr_t",
            "std::unique_ptr<Buffer>": "buffer_uptr_t",
            "std::shared_ptr<RenderContextFrameData>": "rcfd_ptr_t",
            "std::shared_ptr<FxShaderMaterial>": "fxmaterial_ptr_t",
            "std::shared_ptr<FxShaderParam>": "fxparam_ptr_t",
            "std::shared_ptr<GfxMaterial>": "material_ptr_t",
            "std::shared_ptr<Texture>": "texture_ptr_t",
            "std::shared_ptr<RenderTarget>": "rtbuffer_ptr_t",
            "std::shared_ptr<VertexBuffer>": "vtxbuf_ptr_t",
            "std::shared_ptr<IndexBuffer>": "idxbuf_ptr_t",
            "std::shared_ptr<DrawableBuffer>": "drawbuffer_ptr_t",
            # Common STL typedefs
            "std::vector<std::string>": "string_vector_t",
            "std::unordered_map<std::string, std::string>": "string_map_t",
            "std::unordered_map<std::string, int>": "string_int_map_t",
        }
        
        # Apply known typedefs
        for full_type, typedef in common_typedefs.items():
            if full_type in type_str:
                type_str = type_str.replace(full_type, typedef)
        
        # Simplify std:: types when unambiguous
        std_simplifications = {
            "std::string": "string",
            "std::vector": "vector",
            "std::shared_ptr": "shared_ptr",
            "std::unique_ptr": "unique_ptr",
            "std::map": "map",
            "std::unordered_map": "unordered_map",
        }
        
        # Only apply std:: simplifications if not ambiguous
        # (this is a conservative approach, can be made more aggressive)
        
        return type_str
    
    def set_typedef_cache(self, typedef_map: Dict[str, str]):
        """Set the typedef cache for type resolution"""
        self.typedef_cache = typedef_map
        
    def set_namespace_context(self, namespace: str):
        """Set the current namespace context for relative naming"""
        self.namespace_context = namespace

def format_entity_for_display(entity, formatter: Optional[CppFormatter] = None) -> str:
    """
    Main entry point for formatting any C++ entity for display
    """
    if formatter is None:
        formatter = CppFormatter()
        
    # Set namespace context
    if hasattr(entity, 'namespace'):
        formatter.set_namespace_context(entity.namespace)
    
    # Format based on entity type
    if entity.entity_type == "function" or \
       (hasattr(entity, 'member_type') and entity.member_type == 'method'):
        return formatter.format_function_signature(entity)
    elif entity.entity_type == "typedef":
        # Format typedef
        if hasattr(entity, 'target_type') and entity.target_type:
            return f"typedef {entity.target_type} {entity.name}"
        return entity.name
    elif entity.entity_type == "alias":
        # Format type alias
        if hasattr(entity, 'target_type') and entity.target_type:
            return f"using {entity.name} = {entity.target_type}"
        return entity.name
    else:
        # Default: just return the name
        return entity.name


#!/usr/bin/env python3

import argparse
from obt import systemd
from obt.deco import Deco
from obt import command

def main():
    parser = argparse.ArgumentParser(description="Show journal logs for a systemd service")
    parser.add_argument("service_name", help="Service module name (e.g., ork_logger_http)")
    parser.add_argument("-n", "--lines", type=int, default=50, help="Number of lines to show (default: 50)")
    parser.add_argument("-f", "--follow", action="store_true", help="Follow log output")

    args = parser.parse_args()
    deco = Deco()

    try:
        # Verify service module exists
        svc = systemd.requires(args.service_name)
        info = svc.info()

        print(deco.inf(f"Journal logs for service: {args.service_name}"))

        # Build journalctl command based on service type
        journal_cmd = ["journalctl"]

        # Add --user flag only if NOT a system service
        if not info.get('as_system_service', False):
            journal_cmd.append("--user")

        journal_cmd.extend(["-u", f"{args.service_name}.service"])

        if args.follow:
            journal_cmd.append("-f")
        else:
            journal_cmd.extend(["-n", str(args.lines)])

        journal_cmd.append("--no-pager")

        # Run journalctl
        command.run(journal_cmd)

    except RuntimeError as e:
        print(deco.err(str(e)))
        exit(1)

if __name__ == "__main__":
    main()



###############################################################################
# Orkid Build System
# Copyright 2010-2025, Michael T. Mayers
# email: michael@tweakoz.com
# The Orkid Build System is published under the GPL 2.0 license
# see http://www.gnu.org/licenses/gpl-2.0.html
###############################################################################

class serviceinfo:
    """
    Systemd service module for Orkid HTTP Logger Server.
    Provides metadata and configuration for the ork.logger.httpserver service.
    """
    def __init__(self):
        self._name = "ork_logger_http"
        self._command = "ork.logger.httpserver.py"
        self._description = "Orkid HTTP Logger Service - Receives and displays log messages from Orkid applications"
        self._requires = []  # Hard dependencies (fails if dependency fails)
        self._prefers = []   # Soft dependencies (doesn't fail if missing)
        self._after = []     # Start after these (ordering only)
        self._before = []    # Start before these (ordering only)
        self._requires_deps = []     # OBT dep modules to realize (e.g., ["lua", "vulkan"])
        self._requires_dockers = []  # Docker modules to build (e.g., ["postgres_dev"])
        self._requires_pips = []     # Python packages to pip install (e.g., ["flask", "redis"])
        self._requires_tty = None    # TTY number for graphical daemons (e.g., 7 for /dev/tty7)
        self._as_system_service = False  # Run as system service (requires sudo to install)

    def info(self):
        """
        Return service metadata dictionary.
        """
        return {
            "name": self._name,
            "command": self._command,
            "description": self._description,
            "requires": self._requires,
            "prefers": self._prefers,
            "after": self._after,
            "before": self._before,
            "requires_deps": self._requires_deps,
            "requires_dockers": self._requires_dockers,
            "requires_pips": self._requires_pips,
            "requires_tty": self._requires_tty,
            "as_system_service": self._as_system_service,
        }



#!/usr/bin/env python3
###############################################################################

import mdstat, json, sys
from types import SimpleNamespace
from obt import command, deco as DECO 
import zfslib as zfs

###############################################################################

deco = DECO.Deco()

def convert_to_namespace(d):
  return SimpleNamespace(**{k: convert_to_namespace(v) if isinstance(v, dict) else v for k, v in d.items()})

###############################################################################

WHI = (255,255,255)
YEL = (255,255,0)
RED = (255,0,0)
GRN = (0,255,0)
BLU = (0,0,255)
LBLU = (128,128,255)
CYN = (0,255,255)
MAG = (255,0,255)
ORA = (255,128,0)
PUR = (128,0,128)

DK_WHI = (192,192,192)
DK_YEL = (192,192,0)
DK_RED = (192,0,0)
DK_GRN = (0,192,0)
DK_BLU = (0,0,192)
DK_CYN = (0,192,192)
DK_MAG = (192,0,192)
DK_ORA = (192,64,0)
DK_PUR = (64,0,64)
DK_LBLU = (64,64,128)

###############################################################################

def colstr(index, c1, c2, out_str):
  c = c2 if (index&1) else c1
  return deco.rgbstr(c[0],c[1],c[2], out_str)

###############################################################################

def sepstr(name=""):
  if(name==""):
    sepstr = deco.rgbstr(255,192,128,  "#"*80)
  else:
    name_len = len(name)
    l1 = 77 - name_len
    sepstr = "# " + name + " " + (l1 * "#")
    sepstr = deco.rgbstr(255,192,128,  sepstr)
  print(sepstr)

###############################################################################

sepstr()

###############################################################################

def check_drives():
  index = 0
  drivelist = {}
  cap = command.capture(["lsblk","-o","NAME,TYPE,SIZE"])
  cap_lines = cap.split("\n")
  for line in cap_lines:
    split_line = line.split()
    if len(split_line) > 1:
      #print(split_line)
      if split_line[1] == "disk":
        name = split_line[0]
        size = split_line[2]
        drivelist[name] = size

  for disk in drivelist:
    size = drivelist[disk]
    if "sd" in disk or "nvme" in disk:
      cmd_line = ["sudo","smartctl","-H","/dev/"+disk]
      cap = command.capture(cmd_line)
      cap_lines = cap.split("\n")
      status = "???"
      for line in cap_lines:  
        #print(line)
        if "self-assessment test result: PASSED" in line:
          status = "PASSED"
        elif "SMART Health Status: OK" in line:
          status = "OK"
        elif "FAIL" in line:
          status = "FAIL"
      
      
      colors = [MAG,DK_MAG]
      if status == "PASSED" or status == "OK":
        colors = [GRN,DK_GRN]
      elif status == "FAIL":
        colors = [RED,DK_RED]
      
      out_str = "%s(%s)"%(disk,size)
      drivelist[disk]= colstr(0,colors[0],colors[1],f"{out_str:<24}")
      sys.stdout.write(drivelist[disk] + "   ")
      sys.stdout.flush()
      if index % 4 == 3:
        sys.stdout.write("\n")
        sys.stdout.flush()
      index += 1
  print()
  return drivelist
  
###############################################################################
sepstr( "Checking block device SMART drive status (requires sudo)" )
DRIVE_STATS = check_drives()
sepstr()
###############################################################################

def check_zfs():
  conn = zfs.Connection(host='localhost')
  poolset = conn.load_poolset()
  index = 0
  for pool in poolset:
    
    size_mb = int(pool.get_property("size"))>>20
    free_mb = int(pool.get_property("free"))>>20
    allo_mb = int(pool.get_property("allocated"))>>20
    
    out_line = colstr(index,WHI,DK_WHI,"ZFS.pool: ")+deco.yellow("%10s "%pool.name)

    health = pool.get_property("health")
    C1 = WHI
    C2 = DK_WHI
    if health == "ONLINE":
      C1 = GRN
      C2 = DK_GRN
    else:
      C1 = RED
      C2 = DK_RED
    out_line += colstr(index,WHI,DK_WHI,"health: ")+colstr(index,C1,C2,("%8s "%health))
    #out_line += colstr(index,WHI,DK_WHI,"mountpoint: ")+colstr(index,YEL,DK_YEL,"%10s "%pool.get_property("mountpoint"))
    #out_line += colstr(index,WHI,DK_WHI,"free(MB): ")+colstr(index,YEL,DK_YEL,"%9s "%free_mb)
    #out_line += colstr(index,WHI,DK_WHI,"used(MB): ")+colstr(index,YEL,DK_YEL,"%9s "%allo_mb)
    #out_line += colstr(index,WHI,DK_WHI,"size(MB): ")+colstr(index,YEL,DK_YEL,"%9s "%size_mb)
    out_line += colstr(index,WHI,DK_WHI,"frag: ")+colstr(index,YEL,DK_YEL,"%2s "%pool.get_property("fragmentation"))
    
    # get block device names
    #out_line += colstr(index,WHI,DK_WHI,"devices: ")
    #print(dir(pool.children))
    
    print(out_line)
    index += 1
  
###############################################################################

def dict_to_namespace(d):
  """
  Recursively converts a dictionary to a SimpleNamespace object.
  Lists containing dictionaries are also converted.
  """
  if isinstance(d, dict):
    # Convert the dictionary to SimpleNamespace, recursively converting its values
    return SimpleNamespace(**{k: dict_to_namespace(v) for k, v in d.items()})
  elif isinstance(d, list):
    # If the element is a list, recursively convert its items
    return [dict_to_namespace(item) for item in d]
  else:
    # If it's neither a dict nor a list, return the element as is
    return d
  
###############################################################################

def check_mdstat():
  a = mdstat.parse()
  obj = dict_to_namespace(a)
  #print(obj)
  #print(obj.devices)
  index = 0
  for mdev_name in vars(obj.devices):
    #print(mdev_name)
    mdev = getattr(obj.devices, mdev_name)
    #print(mdev)
    out_line = colstr(index,WHI,DK_WHI,"MD.device: ")+colstr(index,YEL,DK_YEL,"%10s "%mdev_name)
    out_line += colstr(index,WHI,DK_WHI,"type: ")+colstr(index,YEL,DK_YEL,"%8s "%mdev.personality)
    #out_line += colstr(index,WHI,DK_WHI,"status: ")+colstr(index,YEL,DK_YEL,"%8s "%mdev.status)
    out_line += colstr(index,WHI,DK_WHI,"disks: ")
    for disk in vars(mdev.disks):
      dname = str(disk)
      #device = Device(dname)
      #status = "OK"
      cmd_line = ["sudo","smartctl","-H","/dev/"+dname]
      cap = command.capture(cmd_line)
      cap_lines = cap.split("\n")
      status = "???"
      for line in cap_lines:  
        #print(line)
        if "self-assessment test result: PASSED" in line:
          status = "PASSED"
        elif "SMART Health Status: OK" in line:
          status = "OK"
        elif "FAIL" in line:
          status = "FAIL"
      
      
      colors = [MAG,DK_MAG]
      if status == "PASSED" or status == "OK":
        colors = [GRN,DK_GRN]
      elif status == "FAIL":
        colors = [RED,DK_RED]
      
      out_line += colstr(index,colors[0],colors[1],"%s "%dname)
      # get freebytes of dname via commandline
      #dev_path = "/dev/"+dname
      #cmdline = ["df", dev_path]
      #out_line += command.Command(cmdline,do_log=True,use_shell=True).exec()
      #print(cap)
      # get smart data
      # get disk status
      # get
      #cmdline = ["sudo","smartctl", "-a", "/dev/"+dname]
      #cap = command.capture(cmdline)
      #out_line += cap
    print(out_line)
    index += 1
  
###############################################################################

def filesystems():
  cmdline = ["df", "-m"]
  cap = command.capture(cmdline)
  index = 0
  
  h1 = "MountPoint"
  h2 = "Device"
  h3 = "Capacity (MiB)"
  h4 = "Used (MiB)"
  h5 = "Avail (MiB)"
  h6 = "Usage %"

  header_str = f"{h1:36} {h2:^28} {h3:>16} {h4:>16} {h5:>16} {h6:>8}"
  print(header_str)

  sorted_lines = {}
  for line in cap.split("\n"):
    if index > 0:
      split_line = line.split()
      if(len(split_line) >= 6):
        key = split_line[5]
        sorted_lines[key] = line
    index += 1  

  sorted_keys = sorted(sorted_lines)
  #print(sorted_lines)
  
  index = 0
  for key in sorted_keys:
    line = sorted_lines[key]
    split_line = line.split()
    
    if(len(split_line) >= 6):
      #print(split_line)
      dev = split_line[0]
      cap = int(split_line[1])
      used = int(split_line[2])
      avail = int(split_line[3])
      percent = split_line[4]
      mount = split_line[5]
      #filesystem = split_line[6]
      cap_str = f"{cap:,}"
      used_str = f"{used:,}"
      avail_str = f"{avail:,}"
      
      percent_as_int = int(percent[:-1])
      percent = f"{percent_as_int}"
      out_line = colstr(index,YEL,DK_YEL,"%-32s "%mount)
      out_line += colstr(index,ORA,DK_ORA,"%32s "%dev)
      out_line += colstr(index,MAG,DK_MAG,"%16s "%cap_str)
      out_line += colstr(index,LBLU,DK_LBLU,"%16s "%used_str)
      out_line += colstr(index,CYN,DK_CYN,"%16s "%avail_str)

      ####################################
      pct_colors = []
      if percent_as_int > 90:
        pct_colors = [RED,DK_RED]
      elif percent_as_int > 80:
        pct_colors = [ORA,DK_ORA]
      elif percent_as_int < 30:
        pct_colors = [GRN,DK_GRN]
      else:
        pct_colors = [YEL,DK_YEL]          
      ####################################

      out_line += colstr(index,pct_colors[0],pct_colors[1],"%8s "%percent)
      #out_line += colstr(index,WHI,DK_WHI,"fs: ")+colstr(index,YEL,DK_YEL,"%-5s "%filesystem)
      
      print(out_line)
    index += 1  

###############################################################################

sepstr("ZFS")
check_zfs()
sepstr("MDSTAT")
check_mdstat()
sepstr("MOUNTS")
filesystems()
sepstr()



#!/usr/bin/env python3
"""
Unit tests for cpp_database_v2.py normalized database
"""
import unittest
import tempfile
import os
from pathlib import Path

from obt.cpp_database_v2 import CppDatabaseV2
from obt.cpp_entities_v2 import (
    Entity, Location, Member, Parameter,
    EntityType, LocationType, AccessLevel, MemberType
)
from ork import path as ork_path

class TestCppDatabaseV2(unittest.TestCase):
    """Test normalized C++ database"""
    
    def setUp(self):
        """Create temporary database for each test"""
        self.temp_db = tempfile.NamedTemporaryFile(suffix='.db', delete=False)
        self.db_path = self.temp_db.name
        self.temp_db.close()
        self.db = CppDatabaseV2(self.db_path)
    
    def tearDown(self):
        """Clean up temporary database"""
        try:
            os.unlink(self.db_path)
        except:
            pass
    
    def test_database_creation(self):
        """Test database schema is created correctly"""
        # Check tables exist
        with self.db.connect() as conn:
            cursor = conn.execute(
                """SELECT name FROM sqlite_master 
                   WHERE type='table' 
                   ORDER BY name"""
            )
            tables = [row[0] for row in cursor]
            
            expected_tables = [
                'entities',
                'entity_locations', 
                'entity_members',
                'entity_references',
                'files'
            ]
            
            for table in expected_tables:
                self.assertIn(table, tables)
    
    def test_add_simple_entity(self):
        """Test adding a simple class entity"""
        entity = Entity(
            canonical_name="MyClass",
            short_name="MyClass",
            entity_type=EntityType.CLASS
        )
        
        # Add location
        entity.add_location(Location("/path/file.cpp", 42))
        
        # Add to database
        entity_id = self.db.add_entity(entity)
        self.assertIsNotNone(entity_id)
        self.assertGreater(entity_id, 0)
        
        # Retrieve and verify
        retrieved = self.db.get_entity("MyClass")
        self.assertIsNotNone(retrieved)
        self.assertEqual(retrieved.canonical_name, "MyClass")
        self.assertEqual(retrieved.entity_type, EntityType.CLASS)
        self.assertEqual(len(retrieved.locations), 1)
        self.assertEqual(retrieved.locations[0].line_number, 42)
    
    def test_no_duplicate_entities(self):
        """Test that adding same entity twice doesn't create duplicates"""
        entity1 = Entity("MyClass", "MyClass", EntityType.CLASS)
        entity1.add_location(Location("/path/file.h", 10))
        
        entity2 = Entity("MyClass", "MyClass", EntityType.CLASS)
        entity2.add_location(Location("/path/file.cpp", 50))
        
        # Add both
        id1 = self.db.add_entity(entity1)
        id2 = self.db.add_entity(entity2)
        
        # Should be same ID (updated, not duplicated)
        self.assertEqual(id1, id2)
        
        # Should have both locations
        retrieved = self.db.get_entity("MyClass")
        self.assertEqual(len(retrieved.locations), 2)
        
        # Check no duplicates in database
        with self.db.connect() as conn:
            count = conn.execute(
                "SELECT COUNT(*) FROM entities WHERE canonical_name = ?",
                ("MyClass",)
            ).fetchone()[0]
            self.assertEqual(count, 1)
    
    def test_no_duplicate_template_classes(self):
        """Test that template classes aren't duplicated (the original bug)"""
        # Regular class version
        regular = Entity(
            canonical_name="ork::util::Context",
            short_name="Context",
            entity_type=EntityType.CLASS,
            namespace="ork::util"
        )
        regular.add_location(Location("/inc/Context.h", 42))
        
        # Template version (same location!)
        template = Entity(
            canonical_name="ork::util::Context",
            short_name="Context",
            entity_type=EntityType.CLASS,
            namespace="ork::util",
            is_template=True,
            template_params="<typename T>"
        )
        template.add_location(Location("/inc/Context.h", 42))
        
        # Add both
        self.db.add_entity(regular)
        self.db.add_entity(template)
        
        # Should only have ONE entity
        entities = self.db.search_entities(name="Context", namespace="ork::util")
        self.assertEqual(len(entities), 1)
        
        # And it should be marked as template
        entity = entities[0]
        self.assertTrue(entity.is_template)
        self.assertEqual(entity.template_params, "<typename T>")
    
    def test_entity_with_members(self):
        """Test adding entity with members"""
        entity = Entity("MyClass", "MyClass", EntityType.CLASS)
        
        # Add members
        field = Member("m_data", MemberType.FIELD, data_type="int",
                      access_level=AccessLevel.PRIVATE)
        method = Member("process", MemberType.METHOD, data_type="void",
                       access_level=AccessLevel.PUBLIC, is_virtual=True)
        
        entity.add_member(field)
        entity.add_member(method)
        
        # Add to database
        self.db.add_entity(entity)
        
        # Retrieve and verify
        retrieved = self.db.get_entity("MyClass")
        self.assertEqual(len(retrieved.members), 2)
        
        # Check members
        fields = retrieved.get_members_by_type(MemberType.FIELD)
        self.assertEqual(len(fields), 1)
        self.assertEqual(fields[0].name, "m_data")
        self.assertEqual(fields[0].data_type, "int")
        
        methods = retrieved.get_members_by_type(MemberType.METHOD)
        self.assertEqual(len(methods), 1)
        self.assertEqual(methods[0].name, "process")
        self.assertTrue(methods[0].is_virtual)
    
    def test_no_duplicate_members(self):
        """Test that duplicate members aren't added"""
        entity = Entity("MyClass", "MyClass", EntityType.CLASS)
        
        # Add member
        member1 = Member("foo", MemberType.METHOD)
        entity.add_member(member1)
        self.db.add_entity(entity)
        
        # Try to add duplicate member
        entity2 = Entity("MyClass", "MyClass", EntityType.CLASS)
        member2 = Member("foo", MemberType.METHOD, is_virtual=True)
        entity2.add_member(member2)
        self.db.add_entity(entity2)
        
        # Should still have only one member
        retrieved = self.db.get_entity("MyClass")
        methods = retrieved.get_members_by_type(MemberType.METHOD)
        self.assertEqual(len(methods), 1)
        
        # But it should be updated
        self.assertTrue(methods[0].is_virtual)
    
    def test_function_entity(self):
        """Test adding function entity with parameters"""
        entity = Entity(
            canonical_name="ork::process",
            short_name="process",
            entity_type=EntityType.FUNCTION,
            namespace="ork",
            return_type="int"
        )
        
        # Add parameters
        param1 = Parameter("input", "const std::string&", is_const=True, is_reference=True)
        param2 = Parameter("flags", "int", default_value="0")
        entity.parameters = [param1, param2]
        
        # Add to database
        self.db.add_entity(entity)
        
        # Retrieve and verify
        retrieved = self.db.get_entity("ork::process")
        self.assertEqual(retrieved.return_type, "int")
        self.assertEqual(len(retrieved.parameters), 2)
        self.assertEqual(retrieved.parameters[0].name, "input")
        self.assertTrue(retrieved.parameters[0].is_const)
        self.assertTrue(retrieved.parameters[0].is_reference)
        self.assertEqual(retrieved.parameters[1].default_value, "0")
    
    def test_enum_entity(self):
        """Test adding enum entity with values"""
        entity = Entity(
            canonical_name="Color",
            short_name="Color",
            entity_type=EntityType.ENUM,
            is_enum_class=True,
            underlying_type="uint32_t"
        )
        
        # Add enum values
        entity.add_member(Member("RED", MemberType.ENUM_VALUE, value="0"))
        entity.add_member(Member("GREEN", MemberType.ENUM_VALUE, value="1"))
        entity.add_member(Member("BLUE", MemberType.ENUM_VALUE, value="2"))
        
        # Add to database
        self.db.add_entity(entity)
        
        # Retrieve and verify
        retrieved = self.db.get_entity("Color")
        self.assertTrue(retrieved.is_enum_class)
        self.assertEqual(retrieved.underlying_type, "uint32_t")
        
        enum_values = retrieved.get_members_by_type(MemberType.ENUM_VALUE)
        self.assertEqual(len(enum_values), 3)
        self.assertEqual(enum_values[0].name, "RED")
        self.assertEqual(enum_values[0].value, "0")
    
    def test_search_entities(self):
        """Test searching for entities"""
        # Add various entities
        entities = [
            Entity("MyClass", "MyClass", EntityType.CLASS),
            Entity("YourClass", "YourClass", EntityType.CLASS),
            Entity("MyStruct", "MyStruct", EntityType.STRUCT),
            Entity("process", "process", EntityType.FUNCTION),
            Entity("std::vector", "vector", EntityType.CLASS, namespace="std"),
            Entity("std::string", "string", EntityType.CLASS, namespace="std"),
        ]
        
        for entity in entities:
            self.db.add_entity(entity)
        
        # Search by type
        classes = self.db.search_entities(entity_type="class")
        self.assertEqual(len(classes), 4)  # MyClass, YourClass, std::vector, std::string
        
        # Search by multiple types
        class_struct = self.db.search_entities(entity_type="class,struct")
        self.assertEqual(len(class_struct), 5)  # All classes + MyStruct
        
        # Search by name
        my_class = self.db.search_entities(name="MyClass")
        self.assertEqual(len(my_class), 1)
        self.assertEqual(my_class[0].short_name, "MyClass")
        
        # Search by pattern
        my_entities = self.db.search_entities(pattern="My")
        self.assertEqual(len(my_entities), 2)  # MyClass, MyStruct
        
        # Search by namespace
        std_entities = self.db.search_entities(namespace="std")
        self.assertEqual(len(std_entities), 2)
        
        # Combined search
        std_classes = self.db.search_entities(entity_type="class", namespace="std")
        self.assertEqual(len(std_classes), 2)
    
    def test_delete_entity(self):
        """Test deleting an entity"""
        entity = Entity("MyClass", "MyClass", EntityType.CLASS)
        entity.add_member(Member("foo", MemberType.METHOD))
        entity.add_location(Location("/path/file.cpp", 42))
        
        self.db.add_entity(entity)
        
        # Verify it exists
        self.assertIsNotNone(self.db.get_entity("MyClass"))
        
        # Delete it
        self.assertTrue(self.db.delete_entity("MyClass"))
        
        # Verify it's gone
        self.assertIsNone(self.db.get_entity("MyClass"))
        
        # Verify cascading delete (members and locations should be gone)
        with self.db.connect() as conn:
            members = conn.execute(
                "SELECT COUNT(*) FROM entity_members"
            ).fetchone()[0]
            self.assertEqual(members, 0)
            
            locations = conn.execute(
                "SELECT COUNT(*) FROM entity_locations"
            ).fetchone()[0]
            self.assertEqual(locations, 0)
    
    def test_clear_database(self):
        """Test clearing all data"""
        # Add some entities
        for i in range(5):
            entity = Entity(f"Class{i}", f"Class{i}", EntityType.CLASS)
            entity.add_location(Location(f"/path/file{i}.cpp", i))
            self.db.add_entity(entity)
        
        # Clear database
        self.db.clear_database()
        
        # Verify everything is gone
        stats = self.db.get_statistics()
        self.assertEqual(stats['total_entities'], 0)
        self.assertEqual(stats['total_locations'], 0)
        self.assertEqual(stats['total_members'], 0)
    
    def test_statistics(self):
        """Test getting database statistics"""
        # Add various entity types
        self.db.add_entity(Entity("Class1", "Class1", EntityType.CLASS))
        self.db.add_entity(Entity("Class2", "Class2", EntityType.CLASS))
        self.db.add_entity(Entity("Struct1", "Struct1", EntityType.STRUCT))
        self.db.add_entity(Entity("func1", "func1", EntityType.FUNCTION))
        self.db.add_entity(Entity("Enum1", "Enum1", EntityType.ENUM))
        
        # Add template
        template = Entity("TClass", "TClass", EntityType.CLASS,
                         is_template=True, template_params="<typename T>")
        self.db.add_entity(template)
        
        stats = self.db.get_statistics()
        
        self.assertEqual(stats['total_entities'], 6)
        self.assertEqual(stats['entities_class'], 3)
        self.assertEqual(stats['entities_struct'], 1)
        self.assertEqual(stats['entities_function'], 1)
        self.assertEqual(stats['entities_enum'], 1)
        self.assertEqual(stats['template_entities'], 1)
    
    def test_file_tracking(self):
        """Test file modification tracking"""
        file_path = "/path/to/file.cpp"
        mtime1 = 1000.0
        
        # First update
        hash1 = self.db.update_file_record(file_path, mtime1)
        self.assertIsNotNone(hash1)
        
        # Check not modified (same time)
        self.assertFalse(self.db.is_file_modified(file_path, mtime1))
        
        # Check modified (newer time)
        self.assertTrue(self.db.is_file_modified(file_path, mtime1 + 1))
        
        # Update with new time
        mtime2 = 2000.0
        hash2 = self.db.update_file_record(file_path, mtime2)
        
        # Now old time should show as not modified
        self.assertFalse(self.db.is_file_modified(file_path, mtime1))
        self.assertFalse(self.db.is_file_modified(file_path, mtime2))
    
    def test_complex_entity(self):
        """Test a complex entity with all features"""
        # Create a complex template class
        entity = Entity(
            canonical_name="ork::util::SmartPtr",
            short_name="SmartPtr",
            entity_type=EntityType.CLASS,
            namespace="ork::util",
            is_template=True,
            template_params="<typename T>",
            is_final=True
        )
        
        # Add base classes
        entity.base_classes = ["BasePtr", "Countable"]
        
        # Add multiple locations
        entity.add_location(Location("/inc/SmartPtr.h", 10, 
                                    location_type=LocationType.DECLARATION))
        entity.add_location(Location("/inc/SmartPtr.hpp", 100,
                                    location_type=LocationType.DEFINITION,
                                    has_body=True))
        
        # Add various members
        entity.add_member(Member("m_ptr", MemberType.FIELD, 
                               data_type="T*",
                               access_level=AccessLevel.PRIVATE))
        entity.add_member(Member("m_count", MemberType.FIELD,
                               data_type="std::atomic<int>",
                               access_level=AccessLevel.PRIVATE,
                               is_static=True))
        entity.add_member(Member("SmartPtr", MemberType.CONSTRUCTOR,
                               access_level=AccessLevel.PUBLIC,
                               is_explicit=True))
        entity.add_member(Member("~SmartPtr", MemberType.DESTRUCTOR,
                               access_level=AccessLevel.PUBLIC,
                               is_virtual=True))
        entity.add_member(Member("get", MemberType.METHOD,
                               data_type="T*",
                               access_level=AccessLevel.PUBLIC,
                               is_const=True,
                               is_inline=True))
        entity.add_member(Member("reset", MemberType.METHOD,
                               data_type="void",
                               access_level=AccessLevel.PUBLIC))
        
        # Add to database
        self.db.add_entity(entity)
        
        # Retrieve and verify everything
        retrieved = self.db.get_entity("ork::util::SmartPtr")
        
        self.assertEqual(retrieved.canonical_name, "ork::util::SmartPtr")
        self.assertEqual(retrieved.namespace, "ork::util")
        self.assertTrue(retrieved.is_template)
        self.assertEqual(retrieved.template_params, "<typename T>")
        self.assertTrue(retrieved.is_final)
        self.assertEqual(retrieved.base_classes, ["BasePtr", "Countable"])
        
        # Check locations
        self.assertEqual(len(retrieved.locations), 2)
        primary = retrieved.get_primary_location()
        self.assertEqual(primary.location_type, LocationType.DEFINITION)
        self.assertTrue(primary.has_body)
        
        # Check members
        self.assertEqual(len(retrieved.members), 6)
        
        fields = retrieved.get_members_by_type(MemberType.FIELD)
        self.assertEqual(len(fields), 2)
        
        constructors = retrieved.get_members_by_type(MemberType.CONSTRUCTOR)
        self.assertEqual(len(constructors), 1)
        self.assertTrue(constructors[0].is_explicit)
        
        destructors = retrieved.get_members_by_type(MemberType.DESTRUCTOR)
        self.assertEqual(len(destructors), 1)
        self.assertTrue(destructors[0].is_virtual)
        
        public_members = retrieved.get_members_by_access(AccessLevel.PUBLIC)
        self.assertEqual(len(public_members), 4)  # ctor, dtor, get, reset
    
    def test_real_orkid_classes(self):
        """Test with real Orkid classes that are stable"""
        # Test LockedResource - a template class for thread-safe resources
        locked_resource = Entity(
            canonical_name="ork::LockedResource",
            short_name="LockedResource",
            entity_type=EntityType.CLASS,
            namespace="ork",
            is_template=True,
            template_params="<typename T>"
        )
        locked_resource.add_location(Location(
            str(ork_path.root / "ork.core" / "inc" / "ork" / "kernel" / "concurrent_queue.h"),
            100  # Approximate line number
        ))
        
        # Test mutex - standard synchronization primitive
        mutex = Entity(
            canonical_name="ork::mutex",
            short_name="mutex",
            entity_type=EntityType.CLASS,
            namespace="ork"
        )
        mutex.add_location(Location(
            str(ork_path.root / "ork.core" / "inc" / "ork" / "kernel" / "mutex.h"),
            50  # Approximate line number
        ))
        
        # Test fvec3 - 3D float vector
        fvec3 = Entity(
            canonical_name="ork::fvec3",
            short_name="fvec3",
            entity_type=EntityType.CLASS,
            namespace="ork"
        )
        fvec3.add_location(Location(
            str(ork_path.root / "ork.core" / "inc" / "ork" / "math" / "cvector3.h"),
            200,  # Approximate line number
            location_type=LocationType.DEFINITION,
            has_body=True
        ))
        # Add some typical fvec3 members
        fvec3.add_member(Member("x", MemberType.FIELD, data_type="float", 
                               access_level=AccessLevel.PUBLIC))
        fvec3.add_member(Member("y", MemberType.FIELD, data_type="float",
                               access_level=AccessLevel.PUBLIC))
        fvec3.add_member(Member("z", MemberType.FIELD, data_type="float",
                               access_level=AccessLevel.PUBLIC))
        fvec3.add_member(Member("normalize", MemberType.METHOD, data_type="void",
                               access_level=AccessLevel.PUBLIC))
        fvec3.add_member(Member("length", MemberType.METHOD, data_type="float",
                               access_level=AccessLevel.PUBLIC, is_const=True))
        
        # Add all to database
        self.db.add_entity(locked_resource)
        self.db.add_entity(mutex)
        self.db.add_entity(fvec3)
        
        # Test searching for them
        all_classes = self.db.search_entities(entity_type="class")
        self.assertEqual(len(all_classes), 3)
        
        # Test template search
        templates = []
        for entity in all_classes:
            if entity.is_template:
                templates.append(entity)
        self.assertEqual(len(templates), 1)
        self.assertEqual(templates[0].short_name, "LockedResource")
        
        # Test fvec3 members
        fvec3_retrieved = self.db.get_entity("ork::fvec3")
        self.assertIsNotNone(fvec3_retrieved)
        self.assertEqual(len(fvec3_retrieved.members), 5)
        
        # Check fields
        fields = fvec3_retrieved.get_members_by_type(MemberType.FIELD)
        self.assertEqual(len(fields), 3)
        field_names = [f.name for f in fields]
        self.assertIn("x", field_names)
        self.assertIn("y", field_names)
        self.assertIn("z", field_names)
        
        # Check methods
        methods = fvec3_retrieved.get_members_by_type(MemberType.METHOD)
        self.assertEqual(len(methods), 2)
        
        # Test that paths are correct
        for entity in [locked_resource, mutex, fvec3]:
            loc = entity.get_primary_location()
            self.assertTrue(loc.file_path.startswith(str(ork_path.root)))
    
    def test_orkid_template_specialization(self):
        """Test handling of template specializations like Context<T>"""
        # Base template
        context_template = Entity(
            canonical_name="ork::util::Context",
            short_name="Context",
            entity_type=EntityType.CLASS,
            namespace="ork::util",
            is_template=True,
            template_params="<typename T>"
        )
        context_template.add_location(Location(
            str(ork_path.root / "ork.core" / "inc" / "ork" / "util" / "Context.h"),
            42,
            location_type=LocationType.DEFINITION
        ))
        
        # Specialization
        context_int = Entity(
            canonical_name="ork::util::Context<int>",
            short_name="Context<int>",
            entity_type=EntityType.CLASS,
            namespace="ork::util",
            is_template_specialization=True,
            specialized_from="ork::util::Context"
        )
        context_int.add_location(Location(
            str(ork_path.root / "ork.core" / "inc" / "ork" / "util" / "Context.h"),
            100,
            location_type=LocationType.DEFINITION
        ))
        
        # Add both
        self.db.add_entity(context_template)
        self.db.add_entity(context_int)
        
        # Should have two separate entities
        contexts = self.db.search_entities(pattern="Context", namespace="ork::util")
        self.assertEqual(len(contexts), 2)
        
        # Check specialization relationship
        spec = self.db.get_entity("ork::util::Context<int>")
        self.assertIsNotNone(spec)
        self.assertTrue(spec.is_template_specialization)
        self.assertEqual(spec.specialized_from, "ork::util::Context")

    def test_typedef_resolution(self):
        """Test resolving typedefs to their underlying types"""
        # Create a template class
        vector3 = Entity(
            canonical_name="Vector3",
            short_name="Vector3",
            entity_type=EntityType.STRUCT,
            is_template=True,
            template_params="<typename T>"
        )
        
        # Add some members to Vector3
        vector3.add_member(Member(
            name="x",
            member_type=MemberType.FIELD,
            data_type="T"
        ))
        vector3.add_member(Member(
            name="y", 
            member_type=MemberType.FIELD,
            data_type="T"
        ))
        vector3.add_member(Member(
            name="length",
            member_type=MemberType.METHOD,
            data_type="T"
        ))
        
        # Create a typedef
        fvec3 = Entity(
            canonical_name="fvec3",
            short_name="fvec3",
            entity_type=EntityType.TYPEDEF,
            aliased_type="Vector3<float>",
            is_using_alias=True
        )
        
        # Add both to database
        self.db.add_entity(vector3)
        self.db.add_entity(fvec3)
        
        # Test resolution
        resolved = self.db.resolve_typedef(fvec3)
        self.assertIsNotNone(resolved)
        self.assertEqual(resolved.short_name, "Vector3")
        self.assertEqual(resolved.entity_type, EntityType.STRUCT)
        self.assertTrue(resolved.is_template)
        
        # Test effective members
        fvec3_members = self.db.get_effective_members(fvec3)
        self.assertEqual(len(fvec3_members), 3)
        member_names = [m.name for m in fvec3_members]
        self.assertIn("x", member_names)
        self.assertIn("y", member_names)
        self.assertIn("length", member_names)
        
        # Test that non-typedef returns itself
        resolved_vector = self.db.resolve_typedef(vector3)
        self.assertEqual(resolved_vector, vector3)

    def test_file_tracking(self):
        """Test file tracking for incremental updates"""
        import tempfile
        import time
        
        # Create a test file
        with tempfile.NamedTemporaryFile(mode='w', suffix='.cpp', delete=False) as f:
            test_file = f.name
            f.write("class TestClass {};")
        
        try:
            # Add file to database
            file_hash1 = self.db.add_file(test_file)
            self.assertIsNotNone(file_hash1)
            
            # Check file is tracked
            stats = self.db.get_statistics()
            self.assertEqual(stats['total_files'], 1)
            
            # Check if file is considered modified (should not be)
            mtime = os.path.getmtime(test_file)
            self.assertFalse(self.db.is_file_modified(test_file, mtime))
            
            # Simulate file modification
            time.sleep(0.01)  # Small delay to ensure different mtime
            with open(test_file, 'w') as f:
                f.write("class TestClass { int x; };")
            
            # Check if file is now considered modified
            new_mtime = os.path.getmtime(test_file)
            self.assertTrue(self.db.is_file_modified(test_file, new_mtime))
            
            # Update file record
            file_hash2 = self.db.update_file_record(test_file, new_mtime)
            self.assertIsNotNone(file_hash2)
            self.assertNotEqual(file_hash1, file_hash2)  # Hash should change
            
            # Now file should not be considered modified
            self.assertFalse(self.db.is_file_modified(test_file, new_mtime))
            
        finally:
            # Clean up
            os.unlink(test_file)
    
    def test_incremental_parsing(self):
        """Test that only modified files are reparsed"""
        import tempfile
        import time
        from obt.cpp_parser_v2 import CppParserV2
        
        parser = CppParserV2()
        
        # Create two test files
        with tempfile.NamedTemporaryFile(mode='w', suffix='.hpp', delete=False) as f1:
            file1 = Path(f1.name)
            f1.write("class Class1 {};")
        
        with tempfile.NamedTemporaryFile(mode='w', suffix='.hpp', delete=False) as f2:
            file2 = Path(f2.name)
            f2.write("class Class2 {};")
        
        try:
            # Initial parse of both files
            entities1 = parser.parse_file(file1)
            for entity in entities1:
                self.db.add_entity(entity)
            self.db.add_file(file1)
            
            entities2 = parser.parse_file(file2)
            for entity in entities2:
                self.db.add_entity(entity)
            self.db.add_file(file2)
            
            # Check both files are tracked
            stats = self.db.get_statistics()
            self.assertEqual(stats['total_files'], 2)
            
            # Check which files need reparsing (none should)
            mtime1 = os.path.getmtime(str(file1))
            mtime2 = os.path.getmtime(str(file2))
            self.assertFalse(self.db.is_file_modified(str(file1), mtime1))
            self.assertFalse(self.db.is_file_modified(str(file2), mtime2))
            
            # Modify file1
            time.sleep(0.01)
            file1.write_text("class Class1 { int member; };")
            new_mtime1 = os.path.getmtime(str(file1))
            
            # Check which files need reparsing (only file1)
            self.assertTrue(self.db.is_file_modified(str(file1), new_mtime1))
            self.assertFalse(self.db.is_file_modified(str(file2), mtime2))
            
        finally:
            # Clean up
            file1.unlink()
            file2.unlink()

if __name__ == "__main__":
    unittest.main()


#!/usr/bin/env python3

import zmq
import json
import time
import sys

class DirectZMQClient:
    def __init__(self, host="localhost", port=5555):
        """
        Connect directly to ZMQ server (no SSL)
        This shows how the connection works without SSL proxy
        """
        self.context = zmq.Context()
        self.socket = self.context.socket(zmq.REQ)
        
        endpoint = f"tcp://{host}:{port}"
        print(f"Connecting directly to ZMQ server at {endpoint}")
        self.socket.connect(endpoint)
    
    def send_request(self, command, data=None):
        """Send a request and return the response"""
        request = {"command": command}
        if data is not None:
            request["data"] = data
        
        request_str = json.dumps(request)
        print(f"Sending: {request_str}")
        
        # Send request
        self.socket.send_string(request_str)
        
        # Receive response
        response_str = self.socket.recv_string()
        print(f"Received: {response_str}")
        
        try:
            return json.loads(response_str)
        except json.JSONDecodeError:
            return {"error": "Invalid JSON response", "raw": response_str}
    
    def test_commands(self):
        """Test various server commands"""
        print("\n=== Testing Direct ZMQ Connection ===\n")
        
        # Test ping
        print("1. Testing ping command:")
        response = self.send_request("ping")
        print(f"   Status: {response.get('status')}")
        print(f"   Message: {response.get('message')}")
        print()
        
        # Test echo
        print("2. Testing echo command:")
        test_data = "Hello from direct client!"
        response = self.send_request("echo", test_data)
        print(f"   Status: {response.get('status')}")
        print(f"   Echo: {response.get('message')}")
        print()
        
        # Test info
        print("3. Testing info command:")
        response = self.send_request("info")
        print(f"   Status: {response.get('status')}")
        print(f"   Server: {response.get('server')}")
        print(f"   Version: {response.get('version')}")
        print()
        
        # Test unknown command
        print("4. Testing unknown command:")
        response = self.send_request("unknown")
        print(f"   Status: {response.get('status')}")
        print(f"   Message: {response.get('message')}")
        print(f"   Available: {response.get('available_commands')}")
        print()
    
    def cleanup(self):
        print("Cleaning up direct client...")
        self.socket.close()
        self.context.term()

def main():
    if len(sys.argv) > 1:
        host = sys.argv[1]
    else:
        host = "localhost"
    
    # Note: This connects directly to port 5555 (ZMQ server)
    # In docker compose setup, this port is not exposed externally
    # so this client would need to run inside the docker network
    # or you'd need to expose port 5555 for testing
    client = DirectZMQClient(host=host, port=5555)
    
    try:
        client.test_commands()
        print("=== Direct ZMQ Client Test Complete ===")
    except Exception as e:
        print(f"Error: {e}")
        print("Note: Direct connection requires ZMQ server port 5555 to be accessible")
        print("Either run this inside docker network or expose port 5555 in docker compose")
    finally:
        client.cleanup()

if __name__ == "__main__":
    main() 


"""
C++ Entity Search Module for V2 Database
Provides common search functionality used by both obt and ork search commands
"""

import re
from typing import List, Dict, Any, Optional
from obt.cpp_database_v2 import CppDatabaseV2
from obt.cpp_entities_v2 import Entity, EntityType

def build_search_parameters(args) -> tuple[Dict[str, Any], List[str]]:
    """
    Build search parameters from command line arguments
    Returns: (search_kwargs, function_subtypes)
    """
    search_kwargs = {}
    function_subtypes = []
    
    # Entity type filter with function sub-types
    if args.types:
        # Handle 'objects' alias - expand to 'struct,class'
        if args.types.lower() == 'objects':
            args.types = 'struct,class'
        
        # Check for special 'files' type
        if args.types.lower() == 'files':
            # Signal to use file listing mode
            search_kwargs['files_mode'] = True
            return search_kwargs, []
        
        # Map user-friendly names to EntityType values
        type_map = {
            'class': EntityType.CLASS.value,
            'struct': EntityType.STRUCT.value,
            'enum': EntityType.ENUM.value,
            'function': EntityType.FUNCTION.value,  # Free functions only
            'memberfn': 'memberfn',  # Member functions
            'staticfn': 'staticfn',  # Static member functions
            'allfn': EntityType.FUNCTION.value,  # All functions
            'typedef': EntityType.TYPEDEF.value,
            'alias': 'alias',  # Using aliases
            'union': EntityType.UNION.value,
            'namespace': EntityType.NAMESPACE.value,
            'files': 'files',  # Special type for file listing
        }
        
        types = []
        for t in args.types.split(','):
            t = t.strip().lower()
            if t in type_map:
                if t in ['memberfn', 'staticfn', 'function', 'allfn']:
                    # Track function subtypes for later filtering
                    function_subtypes.append(t)
                    if t == 'allfn' or len(function_subtypes) > 1:
                        types.append(EntityType.FUNCTION.value)
                    else:
                        types.append(EntityType.FUNCTION.value)
                elif t == 'alias':
                    types.append(EntityType.TYPEDEF.value)
                else:
                    types.append(type_map[t])
            else:
                from obt.deco import Deco
                deco = Deco()
                print(f"{deco.red(f'Unknown type: {t}')}")
                import sys
                sys.exit(1)
        
        if len(types) == 1:
            search_kwargs['entity_type'] = types[0]
        elif types:
            # Remove duplicates
            types = list(set(types))
            search_kwargs['entity_type'] = ','.join(types)
    
    # Name pattern
    if args.pattern:
        if hasattr(args, 'exact') and args.exact:
            search_kwargs['name'] = args.pattern
        else:
            search_kwargs['pattern'] = args.pattern
    
    # Namespace filter
    if hasattr(args, 'namespace') and args.namespace:
        search_kwargs['namespace'] = args.namespace
    
    # Limit (0 means unlimited, don't pass it to search)
    if hasattr(args, 'limit') and args.limit > 0:
        search_kwargs['limit'] = args.limit
    
    return search_kwargs, function_subtypes

def filter_results(results: List[Entity], args, function_subtypes: List[str]) -> List[Entity]:
    """
    Apply additional filtering to search results
    """
    # Filter by function subtype if specified
    if function_subtypes and 'allfn' not in function_subtypes:
        filtered = []
        for entity in results:
            if entity.entity_type == EntityType.FUNCTION:
                if 'function' in function_subtypes and not entity.is_method:
                    filtered.append(entity)
                elif 'memberfn' in function_subtypes and entity.is_method and not entity.is_static:
                    filtered.append(entity)
                elif 'staticfn' in function_subtypes and entity.is_method and entity.is_static:
                    filtered.append(entity)
            else:
                # Non-function entities pass through
                filtered.append(entity)
        results = filtered
    
    # Filter for aliases if specified
    if hasattr(args, 'types') and args.types and 'alias' in args.types:
        results = [e for e in results if e.entity_type == EntityType.TYPEDEF and e.is_using_alias]
    
    # Additional filtering based on args
    if hasattr(args, 'templates_only') and args.templates_only:
        results = [e for e in results if e.is_template]
    
    if hasattr(args, 'derived_only') and args.derived_only:
        results = [e for e in results if e.base_classes]
    
    # Case-insensitive filtering if pattern provided and case-insensitive mode
    if hasattr(args, 'pattern') and args.pattern and \
       hasattr(args, 'case_insensitive') and args.case_insensitive and \
       not (hasattr(args, 'exact') and args.exact):
        pattern = re.compile(args.pattern, re.IGNORECASE)
        results = [e for e in results if pattern.search(e.short_name)]
    
    return results

def search_database(db: CppDatabaseV2, args):
    """
    Perform a search on the database with given arguments
    Returns either List[Entity] or List[Dict] for files mode
    """
    # Build search parameters
    search_kwargs, function_subtypes = build_search_parameters(args)
    
    # Check for files mode
    if search_kwargs.get('files_mode'):
        # Search for files instead of entities
        if hasattr(args, 'pattern') and args.pattern:
            # Search files by pattern
            return db.search_source_files(args.pattern)
        else:
            # List all files
            limit = getattr(args, 'limit', None)
            return db.list_source_files(limit)
    
    # Special case: searching for derived classes
    if hasattr(args, 'derived_from') and args.derived_from:
        results = db.find_derived_classes(args.derived_from)
    else:
        # Remove files_mode from kwargs if present
        search_kwargs.pop('files_mode', None)
        
        # Perform search
        results = db.search_entities(**search_kwargs)
        
        # Apply additional filtering
        results = filter_results(results, args, function_subtypes)
    
    return results

def format_json_results(results: List[Entity]) -> List[Dict]:
    """
    Format search results as JSON-serializable dictionaries
    """
    json_results = []
    for entity in results:
        json_obj = {
            'name': entity.short_name,
            'canonical_name': entity.canonical_name,
            'type': entity.entity_type.value,
            'namespace': entity.namespace,
            'is_template': entity.is_template,
            'template_params': entity.template_params,
            'locations': [
                {
                    'file': loc.file_path,
                    'line': loc.line_number,
                    'type': loc.location_type.value
                }
                for loc in entity.locations
            ]
        }
        
        if entity.entity_type == EntityType.TYPEDEF:
            json_obj['aliased_type'] = entity.aliased_type
        
        
        json_results.append(json_obj)
    
    return json_results


import subprocess
from obt import path 

def encrypt_string(plain_text, passphrase):
    process = subprocess.Popen(
        ['gpg', '--symmetric', '--cipher-algo', 'AES256', '--passphrase', passphrase, '--batch'],
        stdin=subprocess.PIPE,
        stdout=subprocess.PIPE,
        stderr=subprocess.PIPE
    )
    encrypted_data, error = process.communicate(input=plain_text.encode())

    if process.returncode != 0:
        raise Exception(f"Encryption failed: {error.decode()}")

    return encrypted_data

# Example usage
#plain_text = "This is a secret message."
#passphrase = "yourpassphrase"
#encrypted_string = encrypt_string(plain_text, passphrase)
#print(encrypted_string)

def decrypt_string(encrypted_data, passphrase):
    process = subprocess.Popen(
        ['gpg', '--decrypt', '--passphrase', passphrase, '--batch'],
        stdin=subprocess.PIPE,
        stdout=subprocess.PIPE,
        stderr=subprocess.PIPE
    )
    decrypted_data, error = process.communicate(input=encrypted_data)

    if process.returncode != 0:
        raise Exception(f"Decryption failed: {error.decode()}")

    return decrypted_data.decode()

# Example usage
#decrypted_string = decrypt_string(encrypted_string, passphrase)
#print(decrypted_string)

def encrypt_file(input_file, output_file, passphrase):
    process = subprocess.Popen(
        ['gpg', '--symmetric', '--cipher-algo', 'AES256', '--passphrase', passphrase, '--batch', '--output', output_file],
        stdin=subprocess.PIPE,
        stdout=subprocess.PIPE,
        stderr=subprocess.PIPE
    )
    with open(input_file, 'rb') as f:
        input_data = f.read()

    encrypted_data, error = process.communicate(input=input_data)

    if process.returncode != 0:
        raise Exception(f"Encryption failed: {error.decode()}")

    return output_file

# Example usage
#input_file = 'example.txt'
#output_file = 'example.txt.gpg'
#passphrase = 'yourpassphrase'
#encrypt_file(input_file, output_file, passphrase)

def decrypt_file(input_file, output_file, passphrase):
    p_output = path.Path(output_file)
    if p_output.exists():
      p_output.unlink()
    process = subprocess.Popen(
        ['gpg', '--decrypt', '--passphrase', passphrase, '--batch', '--output', output_file],
        stdin=subprocess.PIPE,
        stdout=subprocess.PIPE,
        stderr=subprocess.PIPE
    )
    with open(input_file, 'rb') as f:
        encrypted_data = f.read()

    decrypted_data, error = process.communicate(input=encrypted_data)

    if process.returncode != 0:
        raise Exception(f"Decryption failed: {error.decode()}")

    return output_file

# Example usage
#input_file = 'example.txt.gpg'
#output_file = 'example_decrypted.txt'
#passphrase = 'yourpassphrase'
#decrypt_file(input_file, output_file, passphrase)



import obt.env
import obt.host
import obt.path
import obt.deco
import obt.command
import os, sys, re, json, site

deco = obt.deco.Deco()


def validate_prompt(ps1_string):

  # check for balanced \[ and \]
  open_count = ps1_string.count('\\[')
  close_count = ps1_string.count('\\]')
  if open_count != close_count:
    print(f"Mismatch detected: {open_count} '\\[' found and {close_count} '\\]' found.")
    assert(False)

  # check that all ANSI escape sequences are inside \[\] pairs
  ansi_outside = re.findall(r"(?<!\\\[)(\033\[[^m]*m)(?!\\\])", ps1_string)
  if ansi_outside:
    print(f"Found ANSI escape sequences outside of '\\[' and '\\]': {ansi_outside}")
    assert(False)

##########################################

def root_path():
  import obt.path
  if obt.path.running_from_pip():
    return obt.path.obt_data_base()
  elif "OBT_ROOT" in os.environ:
    return obt.path.Path(os.environ["OBT_ROOT"])
  else:
    return None

class EnvSetup:
  def __init__(self,config):
    assert(config.valid)
    self._config = config

  ###########################################
  def log(self,x):
    if not self._config.quiet:
       print(x)
  ###########################################
  def lazyMakeDirs(self):
    self.log(deco.bright("Making required directories"))
    (obt.path.prefix()/"lib").mkdir(parents=True,exist_ok=True)
    (obt.path.prefix()/"bin").mkdir(parents=True,exist_ok=True)
    (obt.path.prefix()/"include").mkdir(parents=True,exist_ok=True)
    (obt.path.prefix()/"sdks").mkdir(parents=True,exist_ok=True)
    (obt.path.prefix()/"tempdir").mkdir(parents=True,exist_ok=True)
    (obt.path.subspace_root()).mkdir(parents=True,exist_ok=True)
    (obt.path.quarantine()).mkdir(parents=True,exist_ok=True)
    obt.path.downloads().mkdir(parents=True,exist_ok=True)
    obt.path.builds().mkdir(parents=True,exist_ok=True)
    obt.path.manifests().mkdir(parents=True,exist_ok=True)
    obt.path.gitcache().mkdir(parents=True,exist_ok=True)
    obt.path.apps().mkdir(parents=True,exist_ok=True)
    obt.path.buildlogs().mkdir(parents=True,exist_ok=True)
    # create symlink to default python
    if not (obt.path.prefix()/"bin/python").exists():
      if obt.host.IsOsx:
        if obt.host.IsAppleSilicon:
          os.system("ln -s /opt/local/bin/python3 %s" % str(obt.path.prefix()/"bin/os-python"))
        else:
          os.system("ln -s /usr/local/bin/python3 %s" % str(obt.path.prefix()/"bin/os-python"))
      else:
        os.system("ln -s /usr/bin/python3 %s" % str(obt.path.prefix()/"bin/os-python"))

  ###########################################
  def genLaunchScript(self,out_path=None,subspace=None):
    numcores = int(os.environ["OBT_NUM_CORES"])

    LAUNCHENV = []
    if self._config._git_ssh_command!=None:
      LAUNCHENV += ['export GIT_SSH_COMMAND="%s";'%self._config._git_ssh_command]

    LAUNCHENV += ["obt.env.launch.py"]
    LAUNCHENV += ["--numcores", numcores]
    LAUNCHENV += ["--stagedir", self._config.stage_dir]
    for item in self._config._project_dirs:
      LAUNCHENV += ["--project", str(item)]

    if subspace!= None:
      LAUNCHENV += ["--subspace", subspace]

    LAUNCHENV += [";\n"]

    f = open(str(out_path), 'w')
    f.write(" ".join(obt.command.procargs(LAUNCHENV)))
    f.close()
    os.system("chmod ugo+x %s"%str(out_path))

  ###########################################
  def genBashRc(self,obtconfig,out_path=None,override_sysprompt=None):
    self.log(deco.bright("Generating bashrc override_sysprompt<%s>"%override_sysprompt))
    bdeco = obt.deco.Deco(bash=True)

    HOMEDIR = obt.path.Path(os.environ["HOME"])

    BASHRC = ""

    ################################################
    # generate prompt
    ################################################

    stackindic = ""
    stacked = False
    if "OBT_STACK" in os.environ:
      stackindic = os.environ["OBT_STACK"]
      stacked = True

    ####################
    # system prompt (leftmost icon in prompt)

    SYSPROM = ""
    if obt.host.IsOsx:    
      SYSPROM = "🍎"
    elif obt.host.IsLinux:    
      SYSPROM = "🐧"

    if "OBT_USE_PROMPT_PREFIX" in os.environ:
      SYSPROM = os.environ["OBT_USE_PROMPT_PREFIX"]

    if override_sysprompt!=None:
      SYSPROM = override_sysprompt

    PROMPT = bdeco.promptL('%s[ %s %s-${OBT_SUBSPACE_PROMPT} ]'%(SYSPROM,stackindic,self._config.project_name))
    PROMPT += bdeco.promptC("\\w")
    PROMPT += bdeco.promptR("[$(parse_git_branch) ]")
    PROMPT += bdeco.bright("> ")

    validate_prompt(PROMPT)

    ################################################
    # The sanity of this is a little debatable.
    #  on the one hand, some of the user's shell customizations are respected
    #  on the other hand this can perturb the build environment in other unexpected ways..
    ################################################

    if (os.path.exists("~/.bashrc")):
      BASHRC += 'source $HOME/.bashrc;\n' # source users's bash setup

    if (os.path.exists("~/.obt-global/.bashrc")):
      BASHRC += 'source $HOME/.obt-global/.bashrc;\n'

    ################################################

    BASHRC += 'parse_git_branch() { git branch 2> /dev/null | grep "*" | sed -e "s/*//";};\n'

    BASHRC += 'export -f parse_git_branch\n'

    BASHRC += "\nexport PS1='%s';\n" % PROMPT
    BASHRC += "alias ls='ls -G';\n"
    BASHRC += "complete -r -v\n"

    #########################################
    # statically defined goto and push methods
    #########################################

    dirs = {
        "root": "${OBT_ROOT}",
        "project": "${OBT_PROJECT_DIR}",
        "deps": "${OBT_ROOT}/obt/modules/dep",
        #"subspace": "${OBT_STAGE}/subspaces/${OBT_SUBSPACE}",
        "stage": "${OBT_STAGE}",
        "builds": "${OBT_STAGE}/builds",
    }

    #########################################
    # dynamic goto and pushd methods
    #  generated from individual deps
    #########################################

    depitems = obt.dep.DepNode.FindWithMethod("env_goto")
    for depitemk in depitems:
      depitem = depitems[depitemk]
      if depitem.supports_host:
        gotos = depitem.env_goto()
        dirs.update(gotos)

    #########################################

    norm_venvpypath = os.path.normpath(str(site.getsitepackages()[0]))

    BASHRC += "obt.goto.scripts() { cd ${OBT_SCRIPTS_DIR}; };" 
    BASHRC += "obt.goto.venv() { cd ${VIRTUAL_ENV}; };" 

    for k in dirs:
        v = dirs[k]
        BASHRC += "obt.goto.%s() { cd %s; };" % (k,v)
        BASHRC += "obt.push.%s() { pushd %s; };" % (k,v)


    #########################################

    for item in obtconfig._bashrc_lines:
      BASHRC += "%s\n" % item

    ################################################
    # add completions.json from ~/.obt-global/completions.json
    ################################################
  
    OBT_GLOBAL = HOMEDIR/".obt-global"

    if (OBT_GLOBAL/"completions.json").exists():
      json_array = json.load(open(str(OBT_GLOBAL/"completions.json")))
      print(json_array)
      for json_item in json_array:
        BASHRC += "source %s\n" % json_item

    ################################################

    obt_completions_inp = obt.path.obt_bin_priv_base()/"_obt_dep_completions.py"
    for item in ["obt.dep.build.py","obt.dep.info.py","obt.dep.status.py"]:
      completions_line = "complete -C %s %s\n" % (str(obt_completions_inp),item)
      BASHRC += completions_line

    ################################################

    obt_completions_shell_inp = obt.path.obt_bin_priv_base()/"_obt_dep_completions_shell.py"
    completions_line = "complete -C %s obt.dep.shell.py\n" % (str(obt_completions_shell_inp))
    BASHRC += completions_line

    ################################################

    obt_completions_inp = obt.path.obt_bin_priv_base()/"_obt_subspace_completions.py"
    for item in ["obt.subspace.build.py","obt.subspace.launch.py"]:
      completions_line = "complete -C %s %s\n" % (str(obt_completions_inp),item)
      BASHRC += completions_line

    ################################################

    if out_path!=None:
      f = open(str(out_path), 'w')
      f.write(BASHRC)
      f.close()

    return BASHRC



#!/usr/bin/env python3
"""
Display enum values for C++ enums.
Orkid-specific tool for exploring enum definitions.
"""

import sys
import json
import re
from pathlib import Path
from typing import List, Dict, Any

# Add OBT to path
sys.path.append(str(Path(__file__).parent.parent.parent))
from obt.path import stage
from obt.deco import Deco
from obt.cpp_database_v2 import CppDatabaseV2
from obt.cpp_entities_v2 import EntityType, MemberType

deco = Deco()


def find_enums_by_pattern(db: CppDatabaseV2, pattern: str) -> List[Dict[str, Any]]:
    """Find enums matching a pattern (supports wildcards)"""
    with db.connect() as conn:
        cursor = conn.cursor()
        
        # Convert wildcard pattern to SQL LIKE pattern
        sql_pattern = pattern.replace('*', '%')
        
        cursor.execute("""
            SELECT id, canonical_name, short_name, namespace, is_enum_class
            FROM entities 
            WHERE entity_type = 'enum' AND canonical_name LIKE ?
            ORDER BY canonical_name
        """, (sql_pattern,))
        
        return [dict(row) for row in cursor.fetchall()]


def display_enum_values_columnar(db: CppDatabaseV2, enum_info: Dict[str, Any]):
    """Display a single enum's values in columnar format"""
    enum_id = enum_info['id']
    canonical_name = enum_info['canonical_name']
    namespace = enum_info['namespace'] or '-'
    is_enum_class = bool(enum_info.get('is_enum_class', False))
    
    # Check if this is a CrcEnum (base type is crc_enum_t)
    is_crc_enum = False
    with db.connect() as conn:
        cursor = conn.cursor()
        # Check if any member has CrcEnum pattern or if base type suggests it
        cursor.execute("""
            SELECT COUNT(*) FROM entity_members 
            WHERE entity_id = ? AND member_type = 'enum_value' AND value IS NULL
        """, (enum_id,))
        null_count = cursor.fetchone()[0]
        cursor.execute("""
            SELECT COUNT(*) FROM entity_members 
            WHERE entity_id = ? AND member_type = 'enum_value'
        """, (enum_id,))
        total_count = cursor.fetchone()[0]
        # If all values are NULL, likely a CrcEnum
        is_crc_enum = (null_count == total_count and total_count > 0)
    
    # Get enum values
    with db.connect() as conn:
        cursor = conn.cursor()
        cursor.execute("""
            SELECT name, value, line_number
            FROM entity_members
            WHERE entity_id = ? AND member_type = 'enum_value'
            ORDER BY line_number
        """, (enum_id,))
        
        values = cursor.fetchall()
    
    if not values:
        return
    
    # Print header for this enum
    print()
    filename_padded = canonical_name.center(148)
    print(deco.bg('grey1') + deco.fg('pink') + filename_padded + deco.reset())
    
    # Column configuration  
    LINE_LENGTH = 148
    name_col_width = 25
    namespace_col_width = 20
    type_col_width = 20
    value_col_width = 35
    hash_col_width = LINE_LENGTH - (name_col_width + namespace_col_width + type_col_width + value_col_width)
    
    # Print column headers with dark background and white text
    header_configs = [
        (name_col_width, 'left'),
        (namespace_col_width, 'left'),
        (type_col_width, 'left'),
        (value_col_width, 'left'),
        (hash_col_width, 'left')
    ]
    
    # Create header texts with black bg and white fg for each column
    header_texts = [
        deco.bg('black') + deco.white('Name'.ljust(name_col_width)),
        deco.bg('black') + deco.white('Namespace'.ljust(namespace_col_width)),
        deco.bg('black') + deco.white('Type'.ljust(type_col_width)),
        deco.bg('black') + deco.white('Value'.ljust(value_col_width)),
        deco.bg('black') + deco.white('Hash'.ljust(hash_col_width))
    ]
    
    # Format the header - concatenate the colored columns
    header_line = ''.join(header_texts)
    print(header_line + deco.reset())
    
    # Determine enum type string
    # TODO: Extract base type from parsing (e.g., : crc_enum_t)
    base_type = ""  # Would need to be captured during parsing
    
    if is_enum_class:
        if base_type:
            type_str = f'enum class : {base_type}'
        else:
            type_str = 'enum class'
    else:
        if base_type:
            type_str = f'enum : {base_type}'
        else:
            type_str = 'enum'
    
    # Check if any value uses CrcEnum (simplified check)
    uses_crc = False  # Would need to track this during parsing
    
    # Display each enum value
    for i, val in enumerate(values):
        name = val['name']
        value = val['value'] if val['value'] else None
        
        # Format value and hash
        hash_str = ""
        hash_hex = ""
        hash_dec = ""
        if value:
            value_str = f"= {value}"
        elif is_crc_enum:
            # CrcEnum macro value
            value_str = f"CrcEnum({name})"
            # Compute CRC32 hash
            import zlib
            crc_value = zlib.crc32(name.encode('utf-8')) & 0xffffffff
            hash_hex = f"0x{crc_value:08X}"
            hash_dec = f"({crc_value})"
        else:
            # Auto-incremented
            value_str = f"(= {i})"
        
        # Color coding
        name_col = deco.yellow(name.ljust(name_col_width))
        namespace_col = deco.magenta(namespace.ljust(namespace_col_width))
        type_col = deco.green(type_str.ljust(type_col_width))
        value_col = deco.cyan(value_str.ljust(value_col_width))
        if hash_hex:
            hash_col = deco.orange(hash_hex) + " " + deco.fg('ora2') + hash_dec + deco.reset()
        else:
            hash_col = ""
        
        # Use formatColumns for aligned output
        column_configs = [
            (name_col_width, 'left'),
            (namespace_col_width, 'left'),
            (type_col_width, 'left'),
            (value_col_width, 'left'),
            (hash_col_width, 'left')
        ]
        column_texts = [name_col, namespace_col, type_col, value_col, hash_col]
        output = deco.formatColumns(column_configs, column_texts)
        
        print(output)


def search_by_hash(db: CppDatabaseV2, hash_value: str, show_json: bool = False):
    """Search for enum values by CRC32 hash"""
    import zlib
    
    # Parse hash value (support hex or decimal)
    if hash_value.startswith('0x') or hash_value.startswith('0X'):
        target_hash = int(hash_value, 16)
    else:
        target_hash = int(hash_value)
    
    results = []
    
    # Get all enums that might be CrcEnum-based
    with db.connect() as conn:
        cursor = conn.cursor()
        cursor.execute("""
            SELECT DISTINCT e.id, e.canonical_name, e.short_name, e.namespace, e.is_enum_class
            FROM entities e
            JOIN entity_members m ON e.id = m.entity_id
            WHERE e.entity_type = 'enum' 
            AND m.member_type = 'enum_value'
            AND m.value IS NULL
            GROUP BY e.id
        """)
        
        for enum_row in cursor.fetchall():
            enum_info = dict(enum_row)
            
            # Get all values for this enum
            cursor2 = conn.cursor()
            cursor2.execute("""
                SELECT name FROM entity_members
                WHERE entity_id = ? AND member_type = 'enum_value'
                ORDER BY line_number
            """, (enum_info['id'],))
            
            for val_row in cursor2.fetchall():
                value_name = val_row['name']
                # Compute CRC32 for this value
                crc_value = zlib.crc32(value_name.encode('utf-8')) & 0xffffffff
                
                if crc_value == target_hash:
                    results.append({
                        'enum': enum_info['canonical_name'],
                        'value': value_name,
                        'hash_hex': f'0x{crc_value:08X}',
                        'hash_dec': crc_value
                    })
    
    if show_json:
        import json
        print(json.dumps({'results': results}, indent=2))
    else:
        if results:
            print(f"/////////////////////////////////////////////////////////////")
            print(f"// Found {len(results)} match{'es' if len(results) != 1 else ''} for hash {hash_value}")
            print(f"/////////")
            print()
            
            for result in results:
                print(f"{deco.yellow(result['enum'])}::{deco.cyan(result['value'])} = {deco.orange(result['hash_hex'])} {deco.fg('ora2')}({result['hash_dec']}){deco.reset()}")
        else:
            print(f"{deco.red(f'No CrcEnum values found with hash: {hash_value}')}")


def display_enums(db: CppDatabaseV2, pattern: str, show_json: bool = False):
    """Display enums matching pattern"""
    
    # Check if pattern has wildcards
    if '*' in pattern or '%' in pattern:
        # Pattern search
        enums = find_enums_by_pattern(db, pattern)
    else:
        # Exact search - try both short and canonical name
        with db.connect() as conn:
            cursor = conn.cursor()
            
            # Try canonical name first if it has ::
            if '::' in pattern:
                cursor.execute("""
                    SELECT id, canonical_name, short_name, namespace, is_enum_class
                    FROM entities 
                    WHERE canonical_name = ? AND entity_type = 'enum'
                """, (pattern,))
            else:
                # Try short name
                cursor.execute("""
                    SELECT id, canonical_name, short_name, namespace, is_enum_class
                    FROM entities 
                    WHERE short_name = ? AND entity_type = 'enum'
                """, (pattern,))
            
            enums = [dict(row) for row in cursor.fetchall()]
    
    if not enums:
        if show_json:
            print(json.dumps({"error": f"No enums matching '{pattern}'"}, indent=2))
        else:
            print(f"{deco.red(f'No enums matching: {pattern}')}")
        return
    
    if show_json:
        # JSON output for all matching enums
        result = {"enums": []}
        
        for enum_info in enums:
            enum_data = {
                "canonical_name": enum_info['canonical_name'],
                "short_name": enum_info['short_name'],
                "namespace": enum_info['namespace'],
                "is_enum_class": bool(enum_info.get('is_enum_class', False)),
                "values": []
            }
            
            # Get values
            with db.connect() as conn:
                cursor = conn.cursor()
                cursor.execute("""
                    SELECT name, value, line_number
                    FROM entity_members
                    WHERE entity_id = ? AND member_type = 'enum_value'
                    ORDER BY line_number
                """, (enum_info['id'],))
                
                for val in cursor.fetchall():
                    enum_data["values"].append({
                        "name": val['name'],
                        "value": val['value'] if val['value'] else None,
                        "line": val['line_number']
                    })
            
            result["enums"].append(enum_data)
        
        print(json.dumps(result, indent=2))
    else:
        # Columnar display
        print("/////////////////////////////////////////////////////////////")
        print(f"// Found {len(enums)} enum{'s' if len(enums) != 1 else ''}")
        print("/////////")
        
        for enum_info in enums:
            display_enum_values_columnar(db, enum_info)


def main():
    """Main entry point"""
    import argparse
    
    parser = argparse.ArgumentParser(
        description='Display enum values for C++ enums in Orkid'
    )
    
    parser.add_argument(
        'pattern',
        nargs='?',
        help='Enum name or pattern (supports wildcards like "ork::lev2::*")'
    )
    
    parser.add_argument(
        '--match',
        help='Pattern to match enums (alternative to positional argument)'
    )
    
    parser.add_argument(
        '--hash',
        help='Search for enum value by hash (hex like 0xE15695B7 or decimal like 3780548023)'
    )
    
    parser.add_argument(
        '--json',
        action='store_true',
        help='Output in JSON format'
    )
    
    parser.add_argument(
        '--all',
        action='store_true',
        help='Show all enums'
    )
    
    args = parser.parse_args()
    
    # Check if Orkid database exists
    db_path = stage() / "cpp_db_v2_orkid.db"
    if not db_path.exists():
        print(f"{deco.red('Orkid database not found!')}")
        print(f"Expected at: {db_path}")
        print(f"Run 'ork.cpp.db.build.py -m <modules>' to build it first")
        sys.exit(1)
    
    # Open database
    try:
        db = CppDatabaseV2(db_path)
        
        # Check if searching by hash
        if args.hash:
            search_by_hash(db, args.hash, args.json)
        else:
            # Determine pattern for regular enum display
            if args.all:
                pattern = '*'
            elif args.match:
                pattern = args.match
            elif args.pattern:
                pattern = args.pattern
            else:
                parser.error('Either provide a pattern, use --match, --hash, or use --all')
            
            display_enums(db, pattern, args.json)
    except Exception as e:
        print(f"{deco.red(f'Error: {e}')}")
        import traceback
        traceback.print_exc()
        sys.exit(1)


if __name__ == '__main__':
    main()


###############################################################################
# Orkid Build System
# Copyright 2010-2020, Michael T. Mayers
# email: michael@tweakoz.com
# The Orkid Build System is published under the GPL 2.0 license
# see http://www.gnu.org/licenses/gpl-2.0.html
###############################################################################
from obt import dep, log, path, host, patch
from obt.command import Command
###############################################################################
class giflib(dep.StdProvider):
  name = "giflib"
  def __init__(self):
    super().__init__(giflib.name)
    self.declareDep("cmake")
    ###########################################
    toolset = "darwin" if self._target.os=="macos" else "gcc"
    ###########################################
    self._builder = self.createBuilder(dep.CustomBuilder)
    self._builder._cleanOnClean = False
    build_dir = path.builds()/giflib.name

    env = {
      "PREFIX": path.stage()
    }
    ###############################
    def patch_makefile():
      if toolset == "darwin":
        p = patch.patcher(self)
        p.patch(self.source_root,"Makefile")
      return True
    ###############################
    make_clean_command = Command([
      "make","-e",
      "all", "install"
      ],
      working_dir=build_dir,
      environment=env)

    make_incr_command = Command([
      "make","-e","install"],
      working_dir=build_dir,
      environment=env)

    self._builder._cleanbuildcommands += [patch_makefile,make_clean_command]
    self._builder._incrbuildcommands += [make_incr_command]
  ########################################################################
  @property
  def _fetcher(self):
    return dep.GithubFetcher(name=giflib.name,
                             repospec="tweakoz/giflib",
                             revision="5.2.1",
                             recursive=False)

  ########################################################################
  def areRequiredSourceFilesPresent(self):
    return (self.source_root/"Makefile").exists()

  def areRequiredBinaryFilesPresent(self):
    return None



#!/usr/bin/env python3 

import os, proctools, argparse, sys
from obt import path, deco
deco = deco.Deco()

os.environ["DOCKER_BUILDKIT"] = "1"

parser = argparse.ArgumentParser(description='ZMQ SSL Example Docker Builder (OBT compatible)')
parser.add_argument('--service', choices=['zmq-server', 'ssl-proxy', 'all'], 
                   default='all', help='Which service to build')

args, unknown_args = parser.parse_known_args()

cmdlist = ["docker", "compose", "build"]

# Add service-specific build if requested
if args.service != 'all':
    cmdlist.append(args.service)

# Add any additional build args from OBT
cmdlist.extend(unknown_args)

proctools.sync_subprocess(cmdlist)




#!/usr/bin/env python3

import os

deplist =  ["pkgconfig","cmake","wget","curl","libtiff","libpng", "git-lfs"]
deplist += ["portaudio","m4","bison","flex","xz", "gnupg", "fmt"]
deplist += ["scons","zlib","tbb", "glew","flac","libsndfile"]
deplist += ["mpfr","openssl","graphviz","doxygen","swig","tcl-tk"]
deplist += ["pyqt5","qt5", "gcc@13", "xxhash", "autoconf", "automake" ]
deplist += ["vulkan-loader", "vulkan-extensionlayer", "vulkan-headers"]
deplist += ["vulkan-tools", "vulkan-profiles", "vulkan-validationlayers", "vulkan-utility-libraries"]
deplist += ["pandoc", "doctest","ncurses","curlpp","libtar", "libsodium"]

depliststr = " ".join(deplist)
print(depliststr)
os.system("brew install %s" % depliststr)
# remove to avoid conflicts with ork.build's ffmpeg
os.system("brew remove ffmpeg gstreamer notcurses") 

os.system("python3 -m pip install --upgrade setuptools")
